/*
 ##############################################################
 # (C) 2016 BAE SYSTEMS
 #
 # Using an External Interface Sample - Replaced legacy app.
 #
 # $Id: CombiLock.c,v 2.1 2017/11/06 15:33:28 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
 */

#include "CombiLock.h"

#include <sktlib/inetAddress.h>
#include <sktlib/datagramPacket.h>
#include <sktlib/datagramSocket.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#ifndef TRUE
#  define FALSE 0
#  define TRUE !FALSE
#endif

#define OPEN    0
#define LOCKED  1
int doorState = OPEN;

int     COMBI =  0000;

int     code = 0, digitCount = 0;

DatagramSocket dispSock;

/*############################################################################################*/

static void unlock();
static int  isCodeValid();
static int  isCodeCorrect();
static int  isLocked();

static void bleep();
static char* toText( int code, int digits );
static int  sendToPanel( char *code );

#define msleep(msecs) usleep((msecs)*1000)

/*############################################################################################*/

int main()
{
	/* Initialise the lock... */
	unlock();
	/*
	 * ...and now run the state machine */
	CombiLock();
}

/*############################################################################################*/

#define BUFMAX 4
void CombiLock()
{
	char buf[BUFMAX];
	int  bytes, i, tryCount = 0;
	DatagramSocket kbdSock  = datagramSocketCreate( 9000, NULL );
	DatagramPacket pkt      = datagramPacketCreate();
	/**/
	dispSock = datagramSocketCreate( 0, NULL );
	/**/
	for(;;){
		pkt->buffer = (unsigned char*)buf;
		pkt->bufferSizeBytes = BUFMAX;
		if(( bytes = datagramSocketReceive( kbdSock, pkt )) >= 1 ){
			switch(doorState){
			case OPEN:
				switch(buf[0]){
					case 'R': case 'r':
						reset();
						break;
					case 'L': case 'l':
						if( isCodeValid() )
							lock();
						reset();
						break;
					case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
						if( digitCount < 4 ){
							addDigit( buf[0] );
						}else{
							reset();
						}
						break;
					default:
						break;
				}
				break;
			case LOCKED:
				switch(buf[0]){
					case 'R': case 'r':
						reset();
						break;
					case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
						if( digitCount < 4 ){
							addDigit( buf[0] );
						}
						if( digitCount /*still*/< 4 ){
							break;
						}else
						if( digitCount == 4 && isCodeCorrect() ){
							unlock();
							reset();
							tryCount = 0;
						}else{
							if( ++tryCount == 3 ){
								sendToPanel( "WAIT" );
								sleep( 10 );
								tryCount = 0;
							}
							reset();
						}
						break;
					default:
						break;
				}
				break;
			}
		}
	}
}

/*############################################################################################*/

void lock()
{
	int i;
	/**/
	COMBI = code;
	doorState = LOCKED;
	for( i = 0; i < 3; i++ ){
		sendToPanel( "----" );
		msleep(250);
		sendToPanel( toText( code, 4 ));
		msleep(250);
	}
	printf( "The door is %s...\n", ( doorState == LOCKED ? "LOCKED" : "OPEN" ));
	msleep(250);/*  while the bolts slide and door locks... */
}

void unlock()
{
	doorState = OPEN;
	printf( "The door is %s...\n", ( doorState == LOCKED ? "LOCKED" : "OPEN" ));
	msleep(250);/*  while the bolts slide and door opens... */
}

void reset()
{
	bleep();
	digitCount = code = 0;
	sendToPanel( "----" );
}

void addDigit( char ch )
{
	code = code * 10 + (int)(ch - '0');
	digitCount++;
	sendToPanel( toText( code, digitCount ));
}

/*############################################################################################*/

int  isCodeValid()
{
	if( 0 <= code && code <= 9999 && digitCount == 4 )
		return TRUE;
	else
		return FALSE;
}

int  isCodeCorrect()
{
	if( code == COMBI )
		return TRUE;
	else
		return FALSE;
}

int isLocked()
{
	return ( doorState == LOCKED ? TRUE : FALSE );
}

/*############################################################################################*/

void bleep()
{
	putchar( '\007' );
}

char* toText( int code, int digits )
{
	static char buf[6];
	switch( digits ){
	case 1:
		sprintf( buf, "---%1d", code );
		break;
	case 2:
		sprintf( buf, "--%02d", code );
		break;
	case 3:
		sprintf( buf, "-%03d", code );
		break;
	case 4:
		sprintf( buf, "%04d", code );
		break;
	}
	return buf;
}

int sendToPanel( char *code )
{
	DatagramPacket pkt = datagramPacketCreate();
	int bytes;
	/**/
	pkt->buffer = code;
	pkt->bufferSizeBytes = 4;
	pkt->address->value = inetAddressGetByName("localhost")->value;
	pkt->port    = 9001;
	/**/
	bytes = datagramSocketSend( dispSock, pkt );
/* 	printf( "code = %s\n", code ); */
	/**/
	datagramPacketDestroy( pkt );
	return bytes;
}

