﻿' ##############################################################
' # (C) 2016 BAE SYSTEMS
' #
' # Using an External Interface Sample - Re-used legacy application
' #
' # $Id: Panel.vb,v 2.1 2017/11/06 15:33:29 ksappleb Exp $
' #
' # Part of the BAE Systems, Rochester, ECOA Samples collection
' #
' # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
' #
' # This software is developed for and on behalf of BAE Systems (Operations)
' # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
' GE Aviation Systems Limited,
' # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
' # and the copyright is owned by BAE Systems (Operations) Limited,
' # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
' # GE Aviation Systems Limited, General Dynamics United Kingdom
' # Limited and Leonardo MW Ltd
' #
' # This software is developed by BAE Systems (Operations)
' # Limited, Electronic Systems, and is the Intellectual
' # Property of BAE Systems (Operations) Limited, Electronic Systems.
' #
' # The information set out in this document is provided solely
' # on an 'as is' basis and the co-developers of this software make
' # no warranties expressed or implied, including no warranties
' # as to completeness, accuracy or fitness for purpose, with respect
' # to any of the information.
' ##############################################################
'
' This program is (a noddy form of) a remote keypad and display for a combination
' lock.  Key entries are sent to the lock, which sends back the currently entered
' code, which is then displayed.
'
' It's a very crude EXAMPLE.  Real-life would require an encrypted (as a minimum)
' data exchange between keypad and lock !!
'
Imports System.Net
Imports System.Threading
Public Class thePanel

    Private lockSock As New Sockets.UdpClient()
    Private lockAddr As IPEndPoint = New IPEndPoint(Dns.GetHostEntry("localhost").AddressList(0), 9000)
    Private dispSock As New Sockets.UdpClient(New IPEndPoint(IPAddress.Any, 9001))
    Private dispThrd As Thread
    Private dispText As String
    '
    ' Because the display socket listener is on a different thread than the GUI, we need a delegate...
    Private Delegate Sub updateDisplay()
    Private updateDisplayDelegate As updateDisplay = New updateDisplay(AddressOf updateDisplayFunc)

    '########################################################################################

    Private Sub updateDisplayFunc()
        ' Copy the stored message to the display
        ' This is valid, because this function is run on the GUI thread...
        TextBox1.Text = dispText
    End Sub

    '########################################################################################

    Private Sub dispLstnr()
        Dim src As New IPEndPoint(IPAddress.Any, 0)
        Dim msg As Byte()
        While (True)
            Try
                ' Check for input and display it
                msg = dispSock.Receive(src) 'This blocks, so this listener must be run on its own thread...
                If msg.Length > 0 And msg(0) <> 0 Then
                    ' Put a stringified version of the message into store
                    dispText = System.Text.Encoding.ASCII.GetString(msg)
                    ' Tell the GUI thread to display the message...
                    Me.Invoke(Me.updateDisplayDelegate) '
                End If
            Catch e As Exception
                Debug.Print(e.ToString)
            End Try
        End While
    End Sub

    '########################################################################################

    Private Sub Panel_Load(sener As Object, e As System.EventArgs) Handles Me.Load
        dispThrd = New Thread(AddressOf dispLstnr)
        dispThrd.IsBackground = True
        dispThrd.Start()
    End Sub

    '########################################################################################

    Private Sub sendToLock(s As String)
        Dim msg As Byte() = System.Text.Encoding.ASCII.GetBytes(s)
        lockSock.Send(msg, msg.Length, lockAddr)
    End Sub

    '########################################################################################

    Private Sub Button0_Click(sender As System.Object, e As System.EventArgs) Handles Button0.Click
        sendToLock("0")
    End Sub

    Private Sub Button1_Click(sender As System.Object, e As System.EventArgs) Handles Button1.Click
        sendToLock("1")
    End Sub
    Private Sub Button2_Click(sender As System.Object, e As System.EventArgs) Handles Button2.Click
        sendToLock("2")
    End Sub

    Private Sub Button3_Click(sender As System.Object, e As System.EventArgs) Handles Button3.Click
        sendToLock("3")
    End Sub

    Private Sub Button4_Click(sender As System.Object, e As System.EventArgs) Handles Button4.Click
        sendToLock("4")
    End Sub

    Private Sub Button5_Click(sender As System.Object, e As System.EventArgs) Handles Button5.Click
        sendToLock("5")
    End Sub

    Private Sub Button6_Click(sender As System.Object, e As System.EventArgs) Handles Button6.Click
        sendToLock("6")
    End Sub

    Private Sub Button7_Click(sender As System.Object, e As System.EventArgs) Handles Button7.Click
        sendToLock("7")
    End Sub

    Private Sub Button8_Click(sender As System.Object, e As System.EventArgs) Handles Button8.Click
        sendToLock("8")
    End Sub

    Private Sub Button9_Click(sender As System.Object, e As System.EventArgs) Handles Button9.Click
        sendToLock("9")
    End Sub

    Private Sub ButtonL_Click(sender As System.Object, e As System.EventArgs) Handles ButtonL.Click
        sendToLock("L")
    End Sub

    Private Sub ButtonR_Click(sender As System.Object, e As System.EventArgs) Handles ButtonR.Click
        sendToLock("R")
    End Sub

End Class
