/*
 ##############################################################
 # (C) 2016 BAE SYSTEMS
 #
 # Using an External Interface Sample - Module implementation
 #
 # $Id: CombiLock_modMain_Im.c,v 2.2 2017/11/06 15:33:28 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
 */

#include "../CombiLock_modMain_Im/CombiLock_modMain_Im.h"

#include <stdio.h>
#include <time.h>

extern int sendToPanel( char *code );

struct timespec WHIR = { 0, 250000000 };// 250 ms

/*############################################################################################*/
// Support code
/*############################################################################################*/

ECOA__boolean8 isValid( ECOA__int32 code, ECOA__int32 digitCount )
{
	if( 0 <= code && code <= 9999 && digitCount == 4 )
		return ECOA__TRUE;
	else
		return ECOA__FALSE;
}

/*############################################################################################*/

char* toText( int code, int digits )
{
	static char buf[6];
	switch( digits ){
	case 1:
		sprintf( buf, "---%1d", code );
		break;
	case 2:
		sprintf( buf, "--%02d", code );
		break;
	case 3:
		sprintf( buf, "-%03d", code );
		break;
	case 4:
		sprintf( buf, "%04d", code );
		break;
	}
	return buf;
}

void addDigit( CombiLock_modMain_Im__context *context, ECOA__char8 ch )
{
	context->user.code = context->user.code * 10 + (int)(ch - '0');
	context->user.digitCount++;
	sendToPanel( toText( context->user.code, context->user.digitCount ));
}

/*############################################################################################*/
// ECOA Lifecycle
/*############################################################################################*/

void CombiLock_modMain_Im__INITIALIZE__received(CombiLock_modMain_Im__context *context)
{
	context->user.doorState = CombiLock__State_OPEN;
	context->user.COMBI = 0000;
	context->user.code = 0;
	context->user.digitCount = 0;
}

void CombiLock_modMain_Im__START__received(CombiLock_modMain_Im__context *context)
{
   /* User Code Here */
}

void CombiLock_modMain_Im__STOP__received(CombiLock_modMain_Im__context *context)
{
   /* User Code Here */
}

void CombiLock_modMain_Im__SHUTDOWN__received(CombiLock_modMain_Im__context *context)
{
   /* User Code Here */
}

/*############################################################################################*/
// ECOA Module Operations
/*############################################################################################*/

void CombiLock_modMain_Im__digit__received(CombiLock_modMain_Im__context *context,
    const ECOA__char8 ch)
{
	if( context->user.doorState == CombiLock__State_OPEN ){
		if( context->user.digitCount < 4 ){
			addDigit( context, ch );
		}else{
			CombiLock_modMain_Im__reset__received( context );
		}
	}else{// if( context->user.doorState == CombiLock__State_LOCKED )
		if( context->user.digitCount < 4 ){
			addDigit( context, ch );
		}
		if( context->user.digitCount /*still*/< 4 ){
			return;
		}else
		if( context->user.digitCount == 4 && (context->user.code == context->user.COMBI )){
			context->user.doorState = CombiLock__State_OPEN;
			printf( "The door is %s...\n", ( context->user.doorState == CombiLock__State_LOCKED ? "LOCKED" : "OPEN" ));
			nanosleep( &WHIR, NULL );/*  while the bolts slide and door opens... */
			CombiLock_modMain_Im__reset__received( context );
			context->user.tryCount = 0;
		}else{
			if( ++(context->user.tryCount) == 3 ){
				sendToPanel( "WAIT" );
				sleep( 10 );
				context->user.tryCount = 0;
			}
			CombiLock_modMain_Im__reset__received( context );
		}
	}
}

/*############################################################################################*/

void CombiLock_modMain_Im__lock__received(CombiLock_modMain_Im__context *context)
{
	if( context->user.doorState == CombiLock__State_OPEN ){
		if( isValid( context->user.code, context->user.digitCount )){
			ECOA__int8 i;
			/**/
			context->user.COMBI = context->user.code;
			context->user.doorState = CombiLock__State_LOCKED;
			for( i = 0; i < 3; i++ ){
				sendToPanel( "----" );
				nanosleep( &WHIR, NULL );
				sendToPanel( toText( context->user.code, 4 ));
				nanosleep( &WHIR, NULL );
			}
			printf( "The door is %s...\n", ( context->user.doorState == CombiLock__State_LOCKED ? "LOCKED" : "OPEN" ));
			nanosleep( &WHIR, NULL );/*  while the bolts slide and door locks... */
		}
		CombiLock_modMain_Im__reset__received( context );
		//
	}else{// if( context->user.doorState == CombiLock__State_LOCKED )
		;// ignore
	}
}

/*############################################################################################*/

void CombiLock_modMain_Im__reset__received(CombiLock_modMain_Im__context *context)
{
	putchar( '\007' ); fflush( stdout );
	context->user.digitCount = context->user.code = 0;
	sendToPanel( "----" );
}
