/*
 * main.c
 *
 *  This is an "enhanced" version of the ECOA generated "main.c".  It does all the things that the standard "main.c"
 *  needs to do for an ECOA system, but also spawns thread(s) and provides functions for the "legacy application interface",
 *  i.e. the two-faced stuff that links the ECOA and non-ECOA bits together.
 *
 *  Created on: 19 Apr 2016
 *      Author: SW_APPLEBY
 */

#include <stdio.h>
#include <unistd.h>

#include <sktlib/inetAddress.h>
#include <sktlib/datagramPacket.h>
#include <sktlib/datagramSocket.h>
#include "posix_apos_binding.h"
#include <pthread.h>

#include "CombiLock_External_Interface.h"

//##############################################################################
// Non-ECOA legacy/legacy-interface code...
//##############################################################################

#define BUFMAX 4
DatagramSocket dispSock;

void* panelListener( void *p )
{
	char buf[BUFMAX];
	int  bytes, i, tryCount = 0;
	DatagramSocket pnlSock;
	DatagramPacket pkt      = datagramPacketCreate();
	/**/
	pnlSock  = datagramSocketCreate( 9000, NULL );
	dispSock = datagramSocketCreate( 0, NULL );
	/**/
	pkt->buffer = (unsigned char*)buf;
	pkt->bufferSizeBytes = BUFMAX;
	/**/
	for(;;){
		if(( bytes = datagramSocketReceive( pnlSock, pkt )) >= 1 ){
			switch(pkt->buffer[0]){
				case 'R': case 'r':
					CombiLock__reset(); // Invoke the ECOA Module Operation
					break;
				case 'L': case 'l':
					CombiLock__lock(); // Invoke the ECOA Module Operation
					break;
				case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
					CombiLock__digit(pkt->buffer[0]); // Invoke the ECOA Module Operation
					break;
				default:
					break;
			}
		}
	}
}

int sendToPanel( char *code ) // Called BY ECOA Module code...
{
	DatagramPacket pkt = datagramPacketCreate();
	int bytes;
	/**/
	pkt->buffer = code;
	pkt->bufferSizeBytes = 4;
	pkt->address->value = inetAddressGetByName("localhost")->value;
	pkt->port    = 9001;
	/**/
	bytes = datagramSocketSend( dispSock, pkt );
/* 	printf( "code = %s\n", code ); */
	/**/
	datagramPacketDestroy( pkt );
	return bytes;
}

//##############################################################################
// This is mostly a standard ECOA main(), but initialises and starts-up the
// active legacy and legacy interface code before going into the ECOA
// PD_Controller...
//##############################################################################
int main()
{
	pthread_t thrdInf;
	//
	posix_apos_binding__Initialise();

	//##############################################################################
	// Initialise/start all non-ECOA active content (legacy app. and interface)
	//##############################################################################
	pthread_create( &thrdInf, NULL, panelListener, NULL );

	//##############################################################################
	// Continue with "normal" ECOA initialisation...
	//##############################################################################
	pdCombiLock_PD_Controller__Initialise();

	printf("Listening for the control panel...\n");

	while (1)
	{
		sleep(10);
	}
	printf("Died!...\n");

}




